#include <stdio.h>
#include <stdlib.h>
#include <mem.h>
#include <debug.ipp>

typedef struct
   {
      uint nc1;   /* 01 00 */
      uint tat;
      byte nc2;   /* 00    */
   }  DDITATv5;

void show2HEX(byte *bData,int iLen,char cLineFeed='\n')
{
   for(int i=0;i<iLen;i++) printf("%02X ",bData[i]);
   printf("%c",cLineFeed);
}

void main(int argc,char *argv[])
{
   if(argc<2){ printf("Usage: %s [DDIfilename].ddi\n"); return; }
   FILE *in=fopen(argv[1],"rb");
   if(in==NULL){printf("File Open Error !!\n"); return; }
   byte H[0x500];
   DDITATv5 tat[160];
   int i,j,k;
   fread(H,0x500,1,in);
   fclose(in);
   printf("Header ID string [ 20] : \"%20.20s\"\n"            ,&H[0x0]);
   printf("Header EOF mark  [  1] : %02X\n"                    ,H[0x14]);
   printf("Reserved  <fixed>[  5] : ");               show2HEX(&H[0x15],5);
   printf("DDI remarks      [ 40] : \"%40.40s\"\n"            ,&H[0x1A]);
   printf("Disksize program#[  1] : %02X\n"                    ,H[0x42]);
   printf("Total track #    [  1] : %02X=%d\n"                 ,H[0x43],H[0x43]);
   printf("Reserved  <fixed>[  2] : ");               show2HEX(&H[0x44],2);

   memcpy(tat,&H[0x46],sizeof(DDITATv5)*160);
   printf("DDI chain ???    [800] :\n");

   uint uiSectorPerTrack=H[0x4AF];            // Disksize string len
   uiSectorPerTrack=H[uiSectorPerTrack+0x4B0+21];
   for(i=0;i<160;i++)
      if(tat[i].nc1!=1) tat[i].tat=999,printf("--- ");
      else  printf("%03d ",tat[i].tat/(uiSectorPerTrack*2));
   printf("\n");
   printf("Separater <fixed>[  1] : %02X\n"     ,H[0x366]);
   printf("DDI track CRC ???[320] :\n");
   for(i=0;i<160;i++) printf("%04X ",*((uint *)&H[0x367+i*2]));
   printf("\n");

   printf("Disk CRC         [  2] : %04X\n",*((uint *)&H[0x4A7]));
   printf("Separater <fixed>[  1] : %02X\n"                    ,H[0x4A9]);
   printf("--------------------- Disk parameter ---------------------\n");
   printf("Disksize program#[  1] : %02X\n"                    ,H[0x4AA]);
   printf("Reserved  <fixed>[  3] : ");               show2HEX(&H[0x4AB],3);
   printf("Reserved  <fixed>[  1] : %02X\n",H[0x4AE]);
   printf("Disksize str len [  1] : %02X\n",H[0x4AF]);i=H[0x4AF];
   printf("Disk size string [%3d] : \"%*.*s\"\n",i,i,i,&H[0x4B0]);
   printf("Disk Size        [  1] : %02X = %s\n",H[i+0x4B0+0],(H[i+0x4B0+0]==3)?"3":"5");
   printf("Tracks per side  [  1] : %02X = %d\n",H[i+0x4B0+1],H[i+0x4B0+1]);
   printf("Number of side   [  1] : %02X = %d\n",H[i+0x4B0+2],H[i+0x4B0+2]);
   printf("Reserved         [ 18] : ");               show2HEX(&H[i+0x4B0+3],18);
   printf("Secter per track [  1] : %02X = %d\n",H[i+0x4B0+21],H[i+0x4B0+21]);
   printf("Reserved<diskpar>[  4] : ");               show2HEX(&H[i+0x4B0+22],4);
   printf("Secter per track [  1] : %02X = %d\n",H[i+0x4B0+26],(k=H[i+0x4B0+26],k));
   printf("Secter Series ???[%3d] : ",k);             show2HEX(&H[i+0x4B0+27],k);
   printf("Separater <fixed>[  1] : %02X\n",  H[i+0x4B0+27+k]);
   printf("parameter end str[  3] : \"%3.3s\"\n",&H[i+0x4B0+27+k+1]);
   printf("Reserved         [%3d] : (00)*%d\n",40-k,40-k);
   printf("Reserved  <fixed>[  6] : ");               show2HEX(&H[0x4F8],6);
   printf("Header CRC ???   [  2] : %04X\n",*((uint *)&H[0x4FE]));
   printf("\n");
   return;
}

/*

  DISKDUPE.DAT ɤXHU


         {                  {        
                               
                      r         C                                                          C             C
                      r        j                                                         y             y
                            p                                                                                                                                                                               
         s          j         5 y                             j    j                                     sb DDI ɤϰϱƦC覡 HHHH                                                                    j
                   X         3                              X    X                                 zwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww{X
                                                                                                                                                                                                     x
EC 03 01 00 00 00 00 04 04 "Auto"   05 28 02 F6 00 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 09 00 02 2A 50 09 01 02 03 04 05 06 07 08 09                                                                                x01
EC 03 01 01 00 00 00 04 04 "320k"   05 28 02 F6 00 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 08 00 02 2A 50 08 01 02 03 04 05 06 07 08                                                                                   x01
EC 03 01 02 00 00 00 04 04 "360k"   05 28 02 F6 00 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 09 00 02 2A 50 09 01 02 03 04 05 06 07 08 09                                                                                x01
EC 03 01 03 00 00 00 04 04*"400k"   05 28 02 F6 00 00 00 01 02 00 01 00 04 00 04 00 EF 03 00 00 00 0A 00 02 0A 0A 0A 01 06 02 07 03 08 04 09 05 0A                                                                             x01
EC 03 01 04 00 00 00 04 04 "720k"   03 50 02 F6 00 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 09 00 02 2A 50 09 01 02 03 04 05 06 07 08 09                                                                                x01
EC 03 01 05 00 00 00 04 04*"800k"   03 50 02 F6 00 00 00 01 02 00 01 00 04 00 04 00 EF 03 00 00 00 0A 00 02 0A 0A 0A 01 06 02 07 03 08 04 09 05 0A                                                                             x01
EC 03 01 06 00 00 00 04 05 "1.2Mb"  05 50 02 F6 01 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 0F 00 02 1B 54 0F 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F                                                              x01
EC 03 01 07 00 00 00 04 06*"1.36Mb" 05 50 02 F6 01 00 00 01 04 00 01 00 04 00 04 00 EF 03 00 00 00 11 00 02 0A 0A 11 01 0A 02 0B 03 0C 04 0D 05 0E 06 0F 07 10 08 11 09                                                        x01
EC 03 01 08 00 00 00 04 05 "1.4Mb"  03 50 02 F6 02 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 12 00 02 1B 6C 12 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12                                                     x01
EC 03 01 09 00 00 00 04 06*"1.68Mb" 03 50 02 F6 02 00 00 01 04 00 01 00 04 00 04 00 EF 03 00 00 00 15 00 02 0A 0A 15 01 0C 02 0D 03 0E 04 0F 05 10 06 11 07 12 08 13 09 14 0A 15 0B                                            01
EC 03 01 0A 00 00 00 04 05 "2.8Mb"  03 50 02 F6 03 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 24 00 02 38 53 24 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F 20 21 22 23 24 01

Reserved         [  5] : 01 00 05 00 28
Reserved         [  4] : 08 A0 00 01
                         x |track #
                         |w disksize program #
Reserved         [  1] : 01
Disk CRC         [  2] : 8360
Reserved         [  7] : 01 08 00 00 00 04 05
                            |w disksize program #
Disk size string [  5] : 1.4Mb
Reserved         [ 75] : 03 50 02 F6 02 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 12 00 02 1B 6C 12 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 C6 3E 
Reserved         [ 75] : 03 50 02 F6 02 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 12 00 02 1B 6C 12 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 FE E1 
Reserved         [ 75] : 03 50 02 F6 02 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 12 00 02 1B 6C 12 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 3D 7B 
Reserved         [ 75] : 03 50 02 F6 02 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 12 00 02 1B 6C 12
01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
00 00 00 00 00 00 3D 7B

Reserved         [  4] : 06 A0 00 01
                         x |track #
                         |w disksize program #
Reserved         [  7] : 01 06 00 00 00 04 05
                            |w disksize program #
Disk size string [  5] : 1.2Mb
Reserved         [ 75] : 05 50 02 F6 01 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 0F 00 02 1B 54 0F 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 4A 2A 
Reserved         [ 75] : 05 50 02 F6 01 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 0F 00 02 1B 54 0F 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 F7 23 
Reserved         [ 75] : 05 50 02 F6 01 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 0F 00 02 1B 54 0F 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 01 45 6E 64 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 1D A9
                         03 50 02 F6 03 00 00 01 00 00 01 00 04 00 04 00 EF 03 00 00 00 24 00 02 38 53 24 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F 10 11 12 13 14 15 16 17 18 19 1A 1B 1C 1D 1E 1F 20 21 22 23 24 01 45 6E 64 00 00 00 00 00 00

       00 01 02 03  04 05 06 07   08 09 0A 0B  0C 0D 0E 0F
       -- -- -- --  -- -- -- --  -- -- -- --  -- -- -- --
00000  4D 53 44 20  49 6D 61 67  65 20 56 65  72 73 69 6F   MSD Image Versio
00010  6E 20 31 20  1A 01 00 05  00 28 32 32  32 32 32 32   n 1 ...(222222
00020  32 32 32 32  32 32 32 32  32 32 32 32  32 32 32 32   2222222222222222
00030  32 32 32 32  32 32 32 32  32 32 32 32  32 32 32 32   2222222222222222
00040  32 32 08 A0  00 01 01 00  00 00 00 01  00 24 00 00   22.......$..
00050  01 00 48 00  00 01 00 6C  00 00 01 00  90 00 00 01   .H...l.....
00060  00 B4 00 00  01 00 D8 00  00 01 00 FC  00 00 01 00   ..........
00070  20 01 00 01  00 44 01 00  01 00 68 01  00 01 00 8C    ..D..h..
00080  01 00 01 00  B0 01 00 01  00 D4 01 00  01 00 F8 01   ......
00090  00 01 00 1C  02 00 01 00  40 02 00 01  00 64 02 00   ....@..d.
000A0  01 00 88 02  00 01 00 AC  02 00 01 00  D0 02 00 01   ......
000B0  00 F4 02 00  01 00 18 03  00 01 00 3C  03 00 01 00   .....<..
000C0  60 03 00 01  00 84 03 00  01 00 A8 03  00 01 00 CC   `......
000D0  03 00 01 00  F0 03 00 01  00 14 04 00  01 00 38 04   ......8
000E0  00 01 00 5C  04 00 01 00  80 04 00 01  00 A4 04 00   ..\.....
000F0  01 00 C8 04  00 01 00 EC  04 00 01 00  10 05 00 01   ......
00100  00 34 05 00  01 00 58 05  00 01 00 7C  05 00 01 00   .4..X..|..
00110  A0 05 00 01  00 C4 05 00  01 00 E8 05  00 01 00 0C   ......
00120  06 00 01 00  30 06 00 01  00 54 06 00  01 00 78 06   ..0..T..x
00130  00 01 00 9C  06 00 01 00  C0 06 00 01  00 E4 06 00   .......
00140  01 00 08 07  00 01 00 2C  07 00 01 00  50 07 00 01   .....,...P..
00150  00 74 07 00  01 00 98 07  00 01 00 BC  07 00 01 00   .t.........
00160  E0 07 00 01  00 04 08 00  01 00 28 08  00 01 00 4C   ......(...L
00170  08 00 01 00  70 08 00 01  00 94 08 00  01 00 B8 08   ...p.......
00180  00 01 00 DC  08 00 01 00  00 09 00 01  00 24 09 00   .........$..
00190  01 00 48 09  00 01 00 6C  09 00 01 00  90 09 00 01   .H...l.....
001A0  00 B4 09 00  01 00 D8 09  00 01 00 FC  09 00 01 00   ..........
001B0  20 0A 00 01  00 44 0A 00  01 00 68 0A  00 01 00 8C     ..D ..h ..
001C0  0A 00 01 00  B0 0A 00 01  00 D4 0A 00  01 00 F8 0A    .. .. ..
001D0  00 01 00 1C  0B 00 01 00  40 0B 00 01  00 64 0B 00   ....@..d.
001E0  01 00 88 0B  00 01 00 AC  0B 00 01 00  D0 0B 00 01   ......
001F0  00 F4 0B 00  01 00 18 0C  00 01 00 3C  0C 00 01 00   .....<..
00200  60 0C 00 01  00 84 0C 00  01 00 A8 0C  00 01 00 CC   `......
00210  0C 00 01 00  F0 0C 00 01  00 14 0D 00  01 00 38 0D   .... ..8
00220  00 01 00 5C  0D 00 01 00  80 0D 00 01  00 A4 0D 00   ..\ .. .. .
00230  01 00 C8 0D  00 01 00 EC  0D 00 01 00  10 0E 00 01   . .. ...
00240  00 34 0E 00  01 00 58 0E  00 01 00 7C  0E 00 01 00   .4..X..|..
00250  A0 0E 00 01  00 C4 0E 00  01 00 E8 0E  00 01 00 0C   ......
00260  0F 00 01 00  30 0F 00 01  00 54 0F 00  01 00 78 0F   ..0..T..x
00270  00 01 00 9C  0F 00 01 00  C0 0F 00 01  00 E4 0F 00   .......
00280  01 00 08 10  00 01 00 2C  10 00 01 00  50 10 00 01   ....,..P.
00290  00 74 10 00  01 00 98 10  00 01 00 BC  10 00 01 00   .t......
002A0  E0 10 00 01  00 04 11 00  01 00 28 11  00 01 00 4C   ....(..L
002B0  11 00 01 00  70 11 00 01  00 94 11 00  01 00 B8 11   ..p....
002C0  00 01 00 DC  11 00 01 00  00 12 00 01  00 24 12 00   .......$.
002D0  01 00 48 12  00 01 00 6C  12 00 01 00  90 12 00 01   .H..l...
002E0  00 B4 12 00  01 00 D8 12  00 01 00 FC  12 00 01 00   .......
002F0  20 13 00 01  00 44 13 00  01 00 68 13  00 01 00 8C    ..D..h..
00300  13 00 01 00  B0 13 00 01  00 D4 13 00  01 00 F8 13   ......
00310  00 01 00 1C  14 00 01 00  40 14 00 01  00 64 14 00   ....@..d.
00320  01 00 88 14  00 01 00 AC  14 00 01 00  D0 14 00 01   ......
00330  00 F4 14 00  01 00 18 15  00 01 00 3C  15 00 01 00   .....<..
00340  60 15 00 01  00 84 15 00  01 00 A8 15  00 01 00 CC   `......
00350  15 00 01 00  F0 15 00 01  00 14 16 00  01 00 38 16   ......8
00360  00 01 00 5C  16 00 01 64  0D 82 42 0F  72 0E 29 61   ..\.d Br)a
00370  6F EF 30 3A  97 80 5F 9E  4E F3 F1 8D  35 42 BB 6F   o0:_N5Bo
00380  A0 05 ED 28  D9 7F 24 96  65 30 53 17  7E A2 28 26   ($e0S~(&
00390  2F 67 9D 88  9C AF 11 25  1B C0 27 D7  05 F3 BD 69   /g%'i
003A0  FB 0C BE D3  40 25 04 D3  1A 83 19 DE  78 69 3E A6   @%.xi>
003B0  3B 19 BD 6D  2E 65 02 30  18 11 D9 C6  0E AF 2E 4A   ;m.e0.J
003C0  67 0B 98 89  A6 E4 66 41  CC 52 E5 3F  AB D9 DC B0   gfAR?ܰ
003D0  61 35 09 37  0B F1 AA 5C  F9 A9 08 25  E9 0E 33 02   a5.7\.%3
003E0  1F C0 AA 76  D1 8E 21 3F  74 2D 2E 3B  47 43 B4 8A   vю!?t-.;GC
003F0  5A F6 53 55  4F 5E 8E 3C  E1 8E 5A 88  1E 2F 9C 24   ZSUO^<Z/$
00400  7D 4D 3A E2  5C A9 32 1F  22 75 E8 49  09 3A E6 4F   }M:\2"uI.:O
00410  F9 7F D9 22  4B 9A F4 A6  35 57 F4 DF  BF 47 ED E9   "K5W߿G
00420  71 EE 2C BB  BD 81 9A 4F  C9 68 AD 60  15 A8 97 06   q,Oh`
00430  8C 3C 71 78  E9 15 FA F8  13 F1 BA 8E  FE 15 D2 9E   <qxҞ
00440  46 62 BC 71  13 41 C6 A3  67 83 D5 FA  A9 4C 23 45   FbqAƣgL#E
00450  8A CC 34 EE  F9 C4 35 BF  C1 D5 62 E6  12 52 DC 9D   45bRܝ
00460  58 9C 5E DD  8D CE CF 48  2A 40 51 16  86 6C DD 07   X^ݍH*@Ql.
00470  2A CD 2A 6B  CC B7 F9 53  6E DF 73 E3  FD 2E 2C 2E   **k̷Sns.,.
00480  24 52 8B CA  CA 00 E1 E0  04 89 3C 89  15 CF 65 B3   $R.<e
00490  EB A2 9F 70  91 E1 44 46  83 FC CA 15  C9 85 A8 94   뢟pDFɅ
004A0  02 7E A6 3F  AA 2C 08 3B  52 01 08 00  00 00 04 05   ~?,.;R....
004B0  31 2E 34 4D  62 03 50 02  F6 02 00 00  01 00 00 01   1.4MbP....
004C0  00 04 00 04  00 EF 03 00  00 00 12 00  02 1B 6C 12   .......l
004D0  01 02 03 04  05 06 07 08  09 0A 0B 0C  0D 0E 0F 10   ...  
004E0  11 12 01 45  6E 64 00 00  00 00 00 00  00 00 00 00   End..........
004F0  00 00 00 00  00 00 00 00  00 00 00 00  00 00 FE E1   ..............
00500  EB 3C 90 4D  53 44 4F 53  35 2E 30 00  02 01 01 00   <MSDOS5.0..
*/
